/*
 * (C) IBM International Business Machines (2002)
 * (A) David Oleszkiewicz 
 * (A) Daniel Stekloff
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <linux/unistd.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include "cpu_affinity.h"
#include "affinity.h"

int sched_set_affinity(pid_t pid, unsigned long *new_mask) 
{
        int fd, rc;
        struct affinity affin = {pid:pid, mask:*new_mask};
        
        if (!new_mask) return -EINVAL;
        if ((fd = open(AFFINITY_DEVICE, O_RDWR)) < 0) {
                fprintf(stderr, "couldn't open device '%s': %s\n", 
                        AFFINITY_DEVICE, strerror(errno));
                exit(errno);
        }
        if ((rc = ioctl(fd, AFFINITY_IOC_SET_AFFINITY, &affin)) < 0) {
                fprintf(stderr, "failed: %s\n", strerror(errno));
                close(fd);
                exit(errno);
        }
        close(fd);
        return 0;
}

int sched_get_affinity(pid_t pid, unsigned long *cpu_mask) 
{
        int fd, rc;
        struct affinity affin = {pid:pid};

        if ((fd = open(AFFINITY_DEVICE, O_RDWR)) < 0) {
                fprintf(stderr, "couldn't open device '%s': %s\n", 
                        AFFINITY_DEVICE, strerror(errno));
                exit(errno);
        }
        if ((rc = ioctl(fd, AFFINITY_IOC_GET_AFFINITY, &affin)) < 0) {
                fprintf(stderr, "failed: %s\n", strerror(errno));
                close(fd);
                exit(errno);
        }
        *cpu_mask = affin.mask;
        close(fd);
        return 0;
}

int sched_set_exclusive(pid_t pid, unsigned long *new_mask) 
{
        int fd, rc;
        struct affinity affin = {pid:pid, mask:*new_mask};
        
        if (!new_mask) return -EINVAL;

        if ((fd = open(AFFINITY_DEVICE, O_RDWR)) < 0) {
                fprintf(stderr, "couldn't open device '%s': %s\n", 
                        AFFINITY_DEVICE, strerror(errno));
                exit(errno);
        }
        if ((rc = ioctl(fd, AFFINITY_IOC_SET_EXCLUSIVE, &affin)) < 0) {
                fprintf(stderr, "failed: %s\n", strerror(errno));
                close(fd);
                exit(errno);
        }
        close(fd);
        return 0;
}

int sched_reset(void) 
{
        int fd, rc;

        if ((fd = open(AFFINITY_DEVICE, O_RDWR)) < 0) {
                fprintf(stderr, "couldn't open device '%s': %s\n", 
                        AFFINITY_DEVICE, strerror(errno));
                exit(errno);
        }
        if ((rc = ioctl(fd, AFFINITY_IOC_RESET, 0x0)) < 0) {
                fprintf(stderr, "failed: %s\n", strerror(errno));
                close(fd);
                exit(errno);
        }
        close(fd);
        return 0;
}

int sched_get_processor(pid_t pid, unsigned long *cpu) 
{
        int fd, rc, i;
        struct affinity affin = {pid:pid};

        if ((fd = open(AFFINITY_DEVICE, O_RDWR)) < 0) {
                fprintf(stderr, "couldn't open device '%s': %s\n", 
                        AFFINITY_DEVICE, strerror(errno));
                exit(errno);
        }
        if ((rc = ioctl(fd, AFFINITY_IOC_RESET, &affin)) < 0) {
                fprintf(stderr, "failed: %s\n", strerror(errno));
                close(fd);
                exit(errno);
        }
        for (i = 0; affin.mask != 0x0; affin.mask >>= 1) {
                i++;
        }
        *cpu = i;
        close(fd);
        return 0;
}
